<?php

declare(strict_types=1);

namespace Dedi\SyliusClicAndPayPlugin\Payum\Action;

use Dedi\SyliusClicAndPayPlugin\Payum\ClicAndPayApi;
use Dedi\SyliusClicAndPayPlugin\Payum\ClicAndPayParams;
use Payum\Core\Action\ActionInterface;
use Payum\Core\ApiAwareInterface;
use Payum\Core\ApiAwareTrait;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\Reply\HttpResponse;
use Payum\Core\Request\Notify;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;
use SM\Factory\FactoryInterface;
use Sylius\Component\Core\Model\PaymentInterface;
use Sylius\Component\Payment\PaymentTransitions;
use Symfony\Component\HttpFoundation\RequestStack;

class NotifyAction implements ActionInterface, ApiAwareInterface, LoggerAwareInterface
{
    use ApiAwareTrait;

    protected LoggerInterface $logger;

    private RequestStack $requestStack;

    private FactoryInterface $stateMachineFactory;

    public function __construct(RequestStack $requestStack, FactoryInterface $stateMachineFactory)
    {
        $this->requestStack = $requestStack;
        $this->stateMachineFactory = $stateMachineFactory;
        $this->apiClass = ClicAndPayApi::class;
    }

    public function execute($request)
    {
        RequestNotSupportedException::assertSupports($this, $request);

        /** @var PaymentInterface $payment */
        $payment = $request->getFirstModel();
        $order = $payment->getOrder();

        // Get last payment if multi failed from Paybox form
        $lastPayment = $order->getLastPayment(PaymentInterface::STATE_NEW);
        if (null != $lastPayment && $lastPayment->getId() !== $payment->getId()) {
            $payment = $lastPayment;
        }

        $status = $this->requestStack->getCurrentRequest()->get(ClicAndPayParams::VADS_TRANS_STATUS, null);
        $stateMachine = $this->stateMachineFactory->get($payment, PaymentTransitions::GRAPH);

        if ($this->api->isAccepted($status)) {
            $stateMachine->apply(PaymentTransitions::TRANSITION_COMPLETE);
        } elseif ($this->api->isAbandoned($status)) {
            $stateMachine->apply(PaymentTransitions::TRANSITION_CANCEL);
        } else {
            $stateMachine->apply(PaymentTransitions::TRANSITION_FAIL);
        }

        $query = $this->requestStack->getCurrentRequest()->getQueryString();
        $this->logger->info(sprintf('[%s] %s', ClicAndPayApi::CODE, self::class), ['response' => $query, 'payment' => $payment->getDetails()]);

        throw new HttpResponse('', 200);
    }

    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    public function supports($request)
    {
        return
            $request instanceof Notify &&
            $request->getFirstModel() instanceof PaymentInterface;
    }
}
