<?php

declare(strict_types=1);

namespace Dedi\SyliusClicAndPayPlugin\Payum;

use Dedi\SyliusClicAndPayPlugin\Payum\Action\CaptureAction;
use Dedi\SyliusClicAndPayPlugin\Payum\Action\ConvertPaymentAction;
use Dedi\SyliusClicAndPayPlugin\Payum\Action\NotifyAction;
use Dedi\SyliusClicAndPayPlugin\Payum\Action\StatusAction;
use Payum\Core\Bridge\Spl\ArrayObject;
use Payum\Core\GatewayFactory;

class ClicAndPayGatewayFactory extends GatewayFactory
{
    protected function populateConfig(ArrayObject $config)
    {
        $config->defaults([
            'payum.factory_name' => 'clic and pay',
            'payum.factory_title' => 'Clic and Pay',

            'payum.service.requestStack' => '@request_stack',
            'payum.service.sm' => '@sm.factory',

            'payum.action.capture' => static function (ArrayObject $config) {
                return new CaptureAction($config['payum.service.requestStack']);
            },
            'payum.action.notify' => static function (ArrayObject $config) {
                return new NotifyAction($config['payum.service.requestStack'], $config['payum.service.sm']);
            },
            'payum.action.status' => static function (ArrayObject $config) {
                return new StatusAction($config['payum.service.requestStack']);
            },
            'payum.action.convert_payment' => new ConvertPaymentAction(),
        ]);

        if (false == $config['payum.api']) {
            $config['payum.default_options'] = [
                'sandbox' => true,
                ClicAndPayParams::VADS_SITE_ID => '',
                ClicAndPayParams::VADS_SITE_KEY => '',
                ClicAndPayParams::VADS_ACTION_MODE => 'INTERACTIVE',
                ClicAndPayParams::VADS_CURRENCY => ClicAndPayParams::DEVISE_EURO,
                ClicAndPayParams::VADS_PAGE_ACTION => 'PAYMENT',
                ClicAndPayParams::VADS_VERSION => ClicAndPayParams::VERSION,

                ClicAndPayParams::IS_MULTI_PAYMENT => false,
                ClicAndPayParams::MULTI_PAYMENT_COUNT => 3,
                ClicAndPayParams::MULTI_PAYMENT_FIRST_PERCENT => 33,
                ClicAndPayParams::MULTI_PAYMENT_PERIOD => 30,
            ];

            $config->defaults($config['payum.default_options']);

            $config['payum.required_options'] = [
                ClicAndPayParams::VADS_SITE_ID,
                ClicAndPayParams::VADS_SITE_KEY,
            ];

            $config['payum.api'] = static function (ArrayObject $config) {
                $config->validateNotEmpty($config['payum.required_options']);

                return new ClicAndPayApi((array) $config, $config['payum.http_client'], $config['httplug.message_factory']);
            };
        }
    }
}
